@@ -1,40 +1,45 @@
-#' @title Check the stationary condition of a given GMVAR model
+#' @title Check the stationary condition of a given GMVAR, StMVAR, or G-StMVAR model
 #'
-#' @description \code{is_stationary} checks the stationarity condition of a GMVAR model.
+#' @description \code{is_stationary} checks the stationarity condition of a GMVAR, StMVAR, or G-StMVAR model.
 #'
 #' @inheritParams loglikelihood_int
 #' @param d the number of time series in the system.
-#' @param params  a real valued vector specifying the parameter values.
+#' @param params a real valued vector specifying the parameter values.
 #'   \describe{
-#'     \item{\strong{For reduced form model:}}{
-#'       Should be size
-#'       \eqn{((M(pd^2+d+d(d+1)/2+1)-1)x1)} and have form \strong{\eqn{\theta}}\eqn{ = }(\strong{\eqn{\upsilon}}\eqn{_{1}},
-#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1}}), where:
+#'     \item{\strong{For reduced form models:}}{
+#'       Should be size \eqn{((M(pd^2+d+d(d+1)/2+2)-M1-1)x1)} and have the form
+#'       \strong{\eqn{\theta}}\eqn{ = }(\strong{\eqn{\upsilon}}\eqn{_{1}},
+#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}\eqn{)}, where
 #'       \itemize{
 #'         \item \strong{\eqn{\upsilon}}\eqn{_{m}} \eqn{ = (\phi_{m,0},}\strong{\eqn{\phi}}\eqn{_{m}}\eqn{,\sigma_{m})}
 #'         \item \strong{\eqn{\phi}}\eqn{_{m}}\eqn{ = (vec(A_{m,1}),...,vec(A_{m,p})}
-#'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M.
+#'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M,
+#'         \item \strong{\eqn{\nu}}\eqn{=(\nu_{M1+1},...,\nu_{M})}
+#'         \item \eqn{M1} is the number of GMVAR type regimes.
 #'       }
 #'     }
-#'     \item{\strong{For structural GMVAR model:}}{
+#'     \item{\strong{For structural model:}}{
 #'       Should have the form
 #'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\phi}}\eqn{_{1},...,}\strong{\eqn{\phi}}\eqn{_{M},
-#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1})}, where
+#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}\eqn{)}, where
 #'       \itemize{
 #'         \item\strong{\eqn{\lambda}}\eqn{_{m}=(\lambda_{m1},...,\lambda_{md})} contains the eigenvalues of the \eqn{m}th mixture component.
 #'       }
 #'     }
 #'   }
-#'   Above, \eqn{\phi_{m,0}} is the intercept parameter, \eqn{A_{m,i}} denotes the \eqn{i}:th coefficient matrix of
-#'   the \eqn{m}:th mixture component, \eqn{\Omega_{m}} denotes the error term covariance matrix of the \eqn{m}:th
-#'   mixture component, and \eqn{\alpha_{m}} is the mixing weight parameter. The \eqn{W} and \eqn{\lambda_{mi}} are
-#'   structural parameters replacing the error term covariance matrices (see Virolainen, 2020). If \eqn{M=1}, \eqn{\alpha_{m}}
-#'   and \eqn{\lambda_{mi}} are dropped.
-#'
-#'   If \code{parametrization=="mean"}, just replace each \eqn{\phi_{m,0}} with the regimewise mean \eqn{\mu_{m}}.
+#'   Above, \eqn{\phi_{m,0}} is the intercept parameter, \eqn{A_{m,i}} denotes the \eqn{i}th coefficient matrix of the \eqn{m}th
+#'   mixture component, \eqn{\Omega_{m}} denotes the error term covariance matrix of the \eqn{m}:th mixture component, and
+#'   \eqn{\alpha_{m}} is the mixing weight parameter. The \eqn{W} and \eqn{\lambda_{mi}} are structural parameters replacing the
+#'   error term covariance matrices (see Virolainen, 2020). If \eqn{M=1}, \eqn{\alpha_{m}} and \eqn{\lambda_{mi}} are dropped.
+#'   If \code{parametrization=="mean"}, just replace each \eqn{\phi_{m,0}} with regimewise mean \eqn{\mu_{m}}.
 #'   \eqn{vec()} is vectorization operator that stacks columns of a given matrix into a vector. \eqn{vech()} stacks columns
 #'   of a given matrix from the principal diagonal downwards (including elements on the diagonal) into a vector.
-#'   The notation is in line with the cited article by KMS (2016) introducing the GMVAR model.
+#'
+#'   In the \strong{GMVAR model}, \eqn{M1=M} and \strong{\eqn{\nu}} is dropped from the parameter vector. In the \strong{StMVAR} model, \eqn{M1=0}.
+#'   In the \strong{G-StMVAR} model, the first \code{M1} regimes are \emph{GMVAR type} and the rest \code{M2} regimes are \emph{StMVAR type}.
+#'   In \strong{StMVAR} and \strong{G-StMVAR} models, the degrees of freedom parameters in \strong{\eqn{\nu}} should be strictly larger than two.
+#'
+#'   The notation is similar to the cited literature.
 #' @param all_boldA 3D array containing the \eqn{((dp)x(dp))} "bold A" matrices related to each mixture component VAR-process,
 #'   obtained from \code{form_boldA}. Will be computed if not given.
 #' @param tolerance Returns \code{FALSE} if modulus of any eigenvalue is larger or equal to \code{1-tolerance}.
@@ -48,6 +53,7 @@
 #' @keywords internal
 
 is_stationary <- function(p, M, d, params, all_boldA=NULL, structural_pars=NULL, tolerance=1e-3) {
+  M <- sum(M)
   if(is.null(all_boldA)) {
     all_A <- pick_allA(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
     all_boldA <- form_boldA(p=p, M=M, d=d, all_A=all_A)
@@ -79,6 +85,8 @@ is_stationary <- function(p, M, d, params, all_boldA=NULL, structural_pars=NULL,
 #'   the error term covariance matrix of any regime has eigenvalues smaller than this, the model is classified
 #'   as not satisfying positive definiteness assumption. Note that if the tolerance is too small, numerical
 #'   evaluation of the log-likelihood might fail and cause error.
+#' @param df_tol the parameter vector is considered to be outside the parameter space if all degrees of
+#'   freedom parameters are not larger than \code{2 + df_tol}.
 #' @details The parameter vector in the argument \code{params} should be unconstrained and it is used for
 #'   structural models only.
 #' @return Returns \code{TRUE} if the given parameter values are in the parameter space and \code{FALSE} otherwise.
@@ -89,11 +97,22 @@ is_stationary <- function(p, M, d, params, all_boldA=NULL, structural_pars=NULL,
 #'          \emph{Journal of Econometrics}, \strong{192}, 485-498.
 #'    \item Virolainen S. 2020. Structural Gaussian mixture vector autoregressive model. Unpublished working
 #'      paper, available as arXiv:2007.04713.
+#'    \item Virolainen S. 2021. Gaussian and Student's t mixture vector autoregressive model. Unpublished working
+#'      paper, available as arXiv:2109.13648.
 #'  }
 #'  @keywords internal
 
-in_paramspace_int <- function(p, M, d, params, all_boldA, alphas, all_Omega, W_constraints=NULL, stat_tol=1e-3, posdef_tol=1e-8) {
+in_paramspace_int <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), all_boldA, alphas, all_Omega,
+                              W_constraints=NULL, stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
 
+  model <- match.arg(model)
+  if(model != "GMVAR") { # Check degrees of freedom parameters for StMVAR and G-StMVAR models
+    all_df <- pick_df(M=M, params=params, model=model)
+    if(any(all_df <= 2 + df_tol) || any(all_df > 1e+5)) {
+      return(FALSE)
+    }
+  }
+  M <- sum(M) # pick_W, pick_lambdas, and is_stationary all just use the number of mixture components, not M1 and M2 separately
   if(!is.null(W_constraints)) {
     W_pars <- pick_W(p=p, M=M, d=d, params=params, structural_pars=list(W=W_constraints))
     # No need to check zero constraints because the zeros are not parametrized
@@ -166,21 +185,23 @@ in_paramspace_int <- function(p, M, d, params, all_boldA, alphas, all_Omega, W_c
 #'   structural_pars=list(W=W_22))
 #' @export
 
-in_paramspace <- function(p, M, d, params, constraints=NULL, same_means=NULL, structural_pars=NULL, stat_tol=1e-3, posdef_tol=1e-8) {
-  check_pMd(p=p, M=M, d=d)
+in_paramspace <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), constraints=NULL, same_means=NULL,
+                          structural_pars=NULL, stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
+  model <- match.arg(model)
+  check_pMd(p=p, M=M, d=d, model=model)
   check_constraints(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
-  if(length(params) != n_params(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
+  if(length(params) != n_params(p=p, M=M, d=d, model=model, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
     stop("The parameter vector has wrong length!")
   }
   W_constraints <- structural_pars$W
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, same_means=same_means,
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, model=model, constraints=constraints, same_means=same_means,
                                     structural_pars=structural_pars)
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   all_A <- pick_allA(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
-  in_paramspace_int(p=p, M=M, d=d, params=params, all_boldA=form_boldA(p=p, M=M, d=d, all_A=all_A),
-                    alphas=pick_alphas(p=p, M=M, d=d, params=params),
+  in_paramspace_int(p=p, M=M, d=d, params=params, model=model, all_boldA=form_boldA(p=p, M=M, d=d, all_A=all_A),
+                    alphas=pick_alphas(p=p, M=M, d=d, params=params, model=model),
                     all_Omega=pick_Omegas(p=p, M=M, d=d, params=params, structural_pars=structural_pars),
-                    W_constraints=W_constraints, stat_tol=stat_tol, posdef_tol=posdef_tol)
+                    W_constraints=W_constraints, stat_tol=stat_tol, posdef_tol=posdef_tol, df_tol=df_tol)
 }
 
 
@@ -229,29 +250,41 @@ in_paramspace <- function(p, M, d, params, constraints=NULL, same_means=NULL, st
 #' }
 #' @export
 
-check_parameters <- function(p, M, d, params, parametrization=c("intercept", "mean"), constraints=NULL, same_means=NULL,
-                             structural_pars=NULL, stat_tol=1e-3, posdef_tol=1e-8) {
+check_parameters <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), parametrization=c("intercept", "mean"),
+                             constraints=NULL, same_means=NULL, structural_pars=NULL,
+                             stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
+  model <- match.arg(model)
+  check_pMd(p=p, M=M, d=d, model=model)
+  if(model != "GMVAR") { # Check degrees of freedom parameters for StMVAR and G-StMVAR models
+    if(any(pick_df(M=M, params=params, model=model) <= 2 + df_tol)) {
+      stop("The degrees of freedom parameters are not strictly larger than two (with large enough numerical tolerance)")
+    } else if(any(pick_df(M=M, params=params, model=model) > 1e+5)) {
+      stop("The degrees of freedom parameters should not be larger than 1e+5 (they cause numerical problems and are useless)")
+    }
+  }
+  M_orig <- M
+  M <- sum(M)
   parametrization <- match.arg(parametrization)
   check_same_means(parametrization=parametrization, same_means=same_means)
-  check_pMd(p=p, M=M, d=d)
-  check_constraints(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
-  if(length(params) != n_params(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
+  check_constraints(p=p, M=M_orig, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
+  if(length(params) != n_params(p=p, M=M_orig, d=d, model=model, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
     stop("The parameter vector has wrong dimension!")
   }
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
+  params <- reform_constrained_pars(p=p, M=M_orig, d=d, params=params, model=model, constraints=constraints,
+                                    same_means=same_means, structural_pars=structural_pars)
   W_constraints <- structural_pars$W
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
-  alphas <- pick_alphas(p=p, M=M, d=d, params=params)
+  alphas <- pick_alphas(p=p, M=M_orig, d=d, params=params, model=model)
 
   if(!is.null(structural_pars)) {
-    W_pars <- pick_W(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
+    W_pars <- pick_W(p=p, M=M_orig, d=d, params=params, structural_pars=structural_pars)
     if(any(W_pars[W_constraints < 0] > -1e-8, na.rm=TRUE)) {
       stop("The W parameter does not satisfy the (strict) negative sign constraints (with large enough numerical tolerance)")
     } else if(any(W_pars[W_constraints > 0] < 1e-8, na.rm=TRUE)) {
       stop("The W parameter does not satisfy the (strict) positive sign constraints (with large enough numerical tolerance)")
     }
     if(M > 1) {
-      lambdas <- pick_lambdas(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
+      lambdas <- pick_lambdas(p=p, M=M_orig, d=d, params=params, structural_pars=structural_pars)
       if(any(lambdas < 1e-8)) {
         stop("The lambda parameters are not strictly positive (with large enough numerical tolerance)")
       }
@@ -286,6 +319,7 @@ check_parameters <- function(p, M, d, params, parametrization=c("intercept", "me
 #' @keywords internal
 
 check_constraints <- function(p, M, d, constraints=NULL, same_means=NULL, structural_pars=NULL) {
+  M <- sum(M)
   if(!is.null(constraints)) {
     if(!is.matrix(constraints) | !is.numeric(constraints)) {
       stop("The argument constraints should be a numeric matrix (or NULL if no constraints should be employed)")
@@ -343,32 +377,43 @@ check_constraints <- function(p, M, d, constraints=NULL, same_means=NULL, struct
 }
 
 
-#' @title Calculate the number of parameters in GMVAR model parameter vector
+#' @title Calculate the number of parameters in a GMVAR, StMVAR, or G-StMVAR model's parameter vector
 #'
 #' @description \code{n_params} calculates the number of parameters in the model.
 #'
 #' @inheritParams check_parameters
-#' @return Returns the number of parameters in parameter vector of the specified GMVAR model.
+#' @return Returns the number of parameters in the parameter vector of the specified
+#'  GMVAR, StMVAR, or G-StMVAR model.
 #' @section Warning:
 #'  No argument checks!
 #' @inherit in_paramspace references
 #' @keywords internal
 
-n_params <- function(p, M, d, constraints=NULL, same_means=NULL, structural_pars=NULL) {
+n_params <- function(p, M, d, model=c("GMVAR", "StMVAR", "G-StMVAR"), constraints=NULL, same_means=NULL, structural_pars=NULL) {
+  model <- match.arg(model)
+  if(model == "GMVAR") {
+    n_df <- 0
+  } else if(model == "StMVAR") {
+    n_df <- M
+  } else { # model == "G-StMVAR"
+    n_df <- M[2]
+    M <- sum(M)
+  }
   if(is.null(same_means)) {
-    less_pars <- 0 # Number of parameters less compared to models without same interecept constraints
+    less_pars <- 0 # Number of parameters less compared to models without same intercept constraints
   } else {
     g <- length(same_means) # Number groups with the same mean parameters
-    less_pars <- d*(M - g) # Number of parameters less compared to models without same interecept constraints
+    less_pars <- d*(M - g) # Number of parameters less compared to models without same intercept constraints
   }
   if(is.null(structural_pars)) {
-    return(ifelse(is.null(constraints), M*(d^2*p + d + d*(d+1)/2 + 1) - 1, M*(d + d*(d + 1)/2 + 1) + ncol(constraints) - 1) - less_pars)
+    ret <- ifelse(is.null(constraints), M*(d^2*p + d + d*(d+1)/2 + 1) - 1, M*(d + d*(d + 1)/2 + 1) + ncol(constraints) - 1) - less_pars
   } else {
     q <- ifelse(is.null(constraints), M*p*d^2, ncol(constraints))
     n_Wpars <- length(Wvec(structural_pars$W))
     r <- ifelse(is.null(structural_pars$C_lambda), d*(M - 1), ncol(structural_pars$C_lambda))
-    return(M*d + q + n_Wpars + r + M - 1 - less_pars)
+    ret <- M*d + q + n_Wpars + r + M - 1 - less_pars
   }
+  ret + n_df
 }
 
 
@@ -386,7 +431,7 @@ check_data <- function(data, p) {
     data <- as.matrix(data)
   }
   if(!is.matrix(data)) {
-    stop("The data must be numeric matrix!")
+    stop("The data must be numeric matrix (possibly a class 'ts' object)!")
   } else {
     if(anyNA(data)) stop("The data contains NA values!")
     if(!is.numeric(data)) stop("The data must be numeric!")
@@ -419,46 +464,56 @@ all_pos_ints <- function(x) {
 #' @return Throws an error if something is wrong.
 #' @keywords internal
 
-check_pMd <- function(p, M, d) {
-  if(!all_pos_ints(c(p, M))) {
-    stop("Arguments p and M have to be positive integers!")
+check_pMd <- function(p, M, d, model=c("GMVAR", "StMVAR", "G-StMVAR")) {
+  model <- match.arg(model)
+  if(model == "G-StMVAR") {
+    if(length(M) != 2 || !all_pos_ints(M)) {
+      stop("For G-StMVAR model, the argument M must a length two vector with positive integer entries")
+    }
+  } else {
+    if(!all_pos_ints(M) || length(M) != 1) {
+      stop("For GMVAR and StMVAR models, the argument M must be a positive integer")
+    }
+  }
+  if(!all_pos_ints(p) || length(p) != 1) {
+    stop("The argument p must be a positive integer!")
   }
   if(!missing(d)) {
     if(d < 2 | d%%1 != 0) {
-      stop("Argument d, number of columns in the data matrix, has to be positive integer larger than one!
+      stop("Argument d, the number of columns in the data matrix, has to be a positive integer larger than one!
            For univariate analysis, use the package 'uGMAR'")
     }
   }
 }
 
 
-#' @title Checks whether the given object has class attribute 'gmvar'
+#' @title Checks whether the given object has class attribute 'gsmvar'
 #'
-#' @description \code{check_gmvar} checks that the object has class attribute 'gmvar'.
+#' @description \code{check_gsmvar} checks that the object has class attribute 'gsmvar'.
 #'
 #' @param object S3 object to be tested
-#' @param object_name what is the name of the object that should of class 'gmvar'?
-#' @return Throws an error if the object doesn't have the class attribute 'gmvar'.
+#' @param object_name what is the name of the object that should of class 'gsmvar'?
+#' @return Throws an error if the object doesn't have the class attribute 'gsmvar'.
 #' @keywords internal
 
-check_gmvar <- function(object, object_name) {
-  if(missing(object_name)) object_name <- "gmvar"
-  if(!any(class(object) == "gmvar")) {
-    stop(paste("The object", object_name, "has to be of class 'gmvar', typically created with the function 'GMVAR' or 'fitGMVAR'"))
+check_gsmvar <- function(object, object_name) {
+  if(missing(object_name)) object_name <- "gsmvar"
+  if(!any(class(object) == "gsmvar")) {
+    stop(paste("The object", object_name, "has to be of class 'gsmvar', typically created with the function 'GSMVAR' or 'fitGSMVAR'"))
   }
 }
 
 
 #' @title Checks whether the given object contains data
 #'
-#' @description \code{check_null_data} checks that the gmvar object has data.
+#' @description \code{check_null_data} checks that the gsmvar object has data.
 #'
-#' @inheritParams simulateGMVAR
-#' @return Throws an error if is.null(gmvar$data).
+#' @inheritParams quantile_residual_tests
+#' @return Throws an error if is.null(gsmvar$data).
 #' @keywords internal
 
-check_null_data <- function(gmvar) {
-  if(is.null(gmvar$data)) {
+check_null_data <- function(gsmvar) {
+  if(is.null(gsmvar$data)) {
     stop("The model has to contain data! Data can be added without parameter estimation with the function 'add_data'")
   }
 }
@@ -483,18 +538,18 @@ check_same_means <- function(parametrization, same_means) {
 #'
 #' @description \code{warn_ar_roots} warns if the model contains near-unit-roots in some regimes
 #'
-#' @inheritParams simulateGMVAR
+#' @inheritParams quantile_residual_tests
 #' @param tol if eigenvalue is closer than \code{tol} to its bound, a warning is thrown
 #' @details Warns if, for some regime, some moduli of "bold A" eigenvalues are larger than \code{1 - tol} or
 #'  some eigenvalue of the error term covariance matrix is smaller than \code{tol}.
 #' @return Doesn't return anything.
 #' @keywords internal
 
-warn_eigens <- function(gmvar, tol=0.002) {
-  boldA_eigens <- get_boldA_eigens(gmvar)
-  omega_eigens <- get_omega_eigens(gmvar)
-  near_nonstat <- vapply(1:gmvar$model$M, function(i1) any(abs(boldA_eigens[,i1]) > 1 - tol), logical(1))
-  near_singular <- vapply(1:gmvar$model$M, function(i1) any(abs(omega_eigens[,i1]) < tol), logical(1))
+warn_eigens <- function(gsmvar, tol=0.002) {
+  boldA_eigens <- get_boldA_eigens(gsmvar)
+  omega_eigens <- get_omega_eigens(gsmvar)
+  near_nonstat <- vapply(1:sum(gsmvar$model$M), function(i1) any(abs(boldA_eigens[,i1]) > 1 - tol), logical(1))
+  near_singular <- vapply(1:sum(gsmvar$model$M), function(i1) any(abs(omega_eigens[,i1]) < tol), logical(1))
   if(any(near_nonstat)) {
     my_string1 <- ifelse(sum(near_nonstat) == 1,
                         paste("Regime", which(near_nonstat),"has near-unit-roots! "),
@@ -510,6 +565,33 @@ warn_eigens <- function(gmvar, tol=0.002) {
     my_string2 <- NULL
   }
   if(any(near_nonstat) || any(near_singular)) {
-    warning(paste0(my_string1, my_string2, "Consider building a model from the next-largest local maximum with the function 'alt_gmvar' by adjusting its argument 'which_largest'."))
+    warning(paste0(my_string1, my_string2, "Consider building a model from the next-largest local maximum with the function 'alt_gsmvar' by adjusting its argument 'which_largest'."))
+  }
+}
+
+
+
+#' @title Warn about large degrees of freedom parameter values
+#'
+#' @description \code{warn_df} warns if the model contains large degrees of freedom parameter values
+#'
+#' @inheritParams quantile_residual_tests
+#' @inheritParams loglikelihood_int
+#' @details Warns if, for some regime, the degrees of freedom parameter value is larger than 100.
+#' @return Doesn't return anything.
+#' @keywords internal
+
+warn_df <- function(gsmvar, p, M, params, model=c("GMVAR", "StMVAR", "G-StMVAR")) {
+  model <- match.arg(model)
+  if(!missing(gsmvar)) { # If the model is given as a class 'gsmvar' object, extract the information from it
+    M <- gsmvar$model$M
+    params <- gsmvar$params
+    model <- gsmvar$model$model
+  }
+  if(model == "StMVAR" || model == "G-StMVAR") { # Check whether there is large df parameter value in some regime
+    all_df <- pick_df(M=M, params=params, model=model)
+    if(any(all_df > 100)) {
+      warning("The model contains overly large degrees of freedom parameters. Consider switching to the appropriate G-StMVAR model by setting the corresponding regimes to GMVAR type with the function 'stmvar_to_gstmvar'.")
+    }
   }
 }
