@@ -1,40 +1,46 @@
 #' @import stats
 #'
-#' @title Compute log-likelihood of a Gaussian mixture vector autoregressive model
+#' @title Compute log-likelihood of a GMVAR, StMVAR, and G-StMVAR models
 #'
-#' @description \code{loglikelihood_int} computes log-likelihood of a GMVAR model.
+#' @description \code{loglikelihood_int} computes log-likelihoodof a GMVAR, StMVAR, and G-StMVAR models.
 #'
 #' @inheritParams in_paramspace_int
 #' @param data a matrix or class \code{'ts'} object with \code{d>1} columns. Each column is taken to represent
-#'  a single time series. \code{NA} values are not supported.
+#'  a univariate time series. \code{NA} values are not supported.
 #' @param p a positive integer specifying the autoregressive order of the model.
-#' @param M a positive integer specifying the number of mixture components.
+#' @param M \describe{
+#'   \item{For \strong{GMVAR} and \strong{StMVAR} models:}{a positive integer specifying the number of mixture components.}
+#'   \item{For \strong{G-StMVAR} models:}{a size (2x1) integer vector specifying the number of \emph{GMVAR type} components \code{M1} in the
+#'    first element and \emph{StMVAR type} components \code{M2} in the second element. The total number of mixture components is \code{M=M1+M2}.}
+#' }
 #' @param params a real valued vector specifying the parameter values.
 #'   \describe{
 #'     \item{\strong{For unconstrained models:}}{
-#'       Should be size \eqn{((M(pd^2+d+d(d+1)/2+1)-1)x1)} and have the form
+#'       Should be size \eqn{((M(pd^2+d+d(d+1)/2+2)-M1-1)x1)} and have the form
 #'       \strong{\eqn{\theta}}\eqn{ = }(\strong{\eqn{\upsilon}}\eqn{_{1}},
-#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1}}), where
+#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}\eqn{)}, where
 #'       \itemize{
 #'         \item \strong{\eqn{\upsilon}}\eqn{_{m}} \eqn{ = (\phi_{m,0},}\strong{\eqn{\phi}}\eqn{_{m}}\eqn{,\sigma_{m})}
 #'         \item \strong{\eqn{\phi}}\eqn{_{m}}\eqn{ = (vec(A_{m,1}),...,vec(A_{m,p})}
-#'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M.
+#'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M,
+#'         \item \strong{\eqn{\nu}}\eqn{=(\nu_{M1+1},...,\nu_{M})}
+#'         \item \eqn{M1} is the number of GMVAR type regimes.
 #'       }
 #'     }
 #'     \item{\strong{For constrained models:}}{
-#'       Should be size \eqn{((M(d+d(d+1)/2+1)+q-1)x1)} and have the form
+#'       Should be size \eqn{((M(d+d(d+1)/2+2)+q-M1-1)x1)} and have the form
 #'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\psi},}
-#'       \eqn{\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1})}, where
+#'       \eqn{\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}), where
 #'       \itemize{
 #'         \item \strong{\eqn{\psi}} \eqn{(qx1)} satisfies (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
-#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}} where \strong{\eqn{C}} is \eqn{(Mpd^2xq)}
+#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}} where \strong{\eqn{C}} is a \eqn{(Mpd^2xq)}
 #'         constraint matrix.
 #'       }
 #'     }
 #'     \item{\strong{For same_means models:}}{
 #'       Should have the form
 #'       \strong{\eqn{\theta}}\eqn{ = (}\strong{\eqn{\mu},}\strong{\eqn{\psi},}
-#'       \eqn{\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1})}, where
+#'       \eqn{\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}\eqn{)}, where
 #'       \itemize{
 #'         \item \strong{\eqn{\mu}}\eqn{= (\mu_{1},...,\mu_{g})} where
 #'           \eqn{\mu_{i}} is the mean parameter for group \eqn{i} and
@@ -44,10 +50,10 @@
 #'           (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}\strong{\eqn{\phi}}\eqn{_{M})}.
 #'       }
 #'     }
-#'     \item{\strong{For structural GMVAR model:}}{
+#'     \item{\strong{For structural models:}}{
 #'       Should have the form
 #'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\phi}}\eqn{_{1},...,}\strong{\eqn{\phi}}\eqn{_{M},
-#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1})}, where
+#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1},}\strong{\eqn{\nu}}\eqn{)}, where
 #'       \itemize{
 #'         \item\strong{\eqn{\lambda}}\eqn{_{m}=(\lambda_{m1},...,\lambda_{md})} contains the eigenvalues of the \eqn{m}th mixture component.
 #'       }
@@ -73,7 +79,14 @@
 #'   If \code{parametrization=="mean"}, just replace each \eqn{\phi_{m,0}} with regimewise mean \eqn{\mu_{m}}.
 #'   \eqn{vec()} is vectorization operator that stacks columns of a given matrix into a vector. \eqn{vech()} stacks columns
 #'   of a given matrix from the principal diagonal downwards (including elements on the diagonal) into a vector.
-#'   The notation is in line with the cited article by \emph{Kalliovirta, Meitz and Saikkonen (2016)} introducing the GMVAR model.
+#'
+#'   In the \strong{GMVAR model}, \eqn{M1=M} and \strong{\eqn{\nu}} is dropped from the parameter vector. In the \strong{StMVAR} model, \eqn{M1=0}.
+#'   In the \strong{G-StMVAR} model, the first \code{M1} regimes are \emph{GMVAR type} and the rest \code{M2} regimes are \emph{StMVAR type}.
+#'   In \strong{StMVAR} and \strong{G-StMVAR} models, the degrees of freedom parameters in \strong{\eqn{\nu}} should be strictly larger than two.
+#'
+#'   The notation is similar to the cited literature.
+#' @param model is "GMVAR", "StMVAR", or "G-StMVAR" model considered? In the G-StMVAR model, the first \code{M1} components
+#'  are GMVAR type and the rest \code{M2} components are StMVAR type.
 #' @param conditional a logical argument specifying whether the conditional or exact log-likelihood function
 #'  should be used.
 #' @param parametrization \code{"intercept"} or \code{"mean"} determining whether the model is parametrized with intercept
@@ -110,28 +123,32 @@
 #'   computation time if it does for sure.
 #' @param minval the value that will be returned if the parameter vector does not lie in the parameter space
 #'   (excluding the identification condition).
-#' @param to_return should the returned object be the log-likelihood value, mixing weights, mixing weights including
-#'   value for \eqn{alpha_{m,T+1}}, a list containing log-likelihood value and mixing weights, or
-#'   the terms \eqn{l_{t}: t=1,..,T} in the log-likelihood function (see \emph{KMS 2016, eq.(9)})? Or should
-#'   the regimewise conditional means, total conditional means, or total conditional covariance matrices
-#'   be returned? Default is the log-likelihood value (\code{"loglik"}).
+#' @param to_return should the returned object be the log-likelihood value, which is default, or something else?
+#'  See the section "Return" for all the options.
 #' @details \code{loglikelihood_int} takes use of the function \code{dmvn} from the package \code{mvnfast}.
 #' @return
-#'  \describe{
-#'   \item{By default:}{log-likelihood value of the specified GMVAR model,}
+#'   \item{By default:}{log-likelihood value of the specified GMVAR, StMVAR, or G-StMVAR model,}
 #'   \item{If \code{to_return=="mw"}:}{a size ((n_obs-p)xM) matrix containing the mixing weights: for m:th component in m:th column.}
 #'   \item{If \code{to_return=="mw_tplus1"}:}{a size ((n_obs-p+1)xM) matrix containing the mixing weights: for m:th component in m:th column.
-#'     The last row is for \eqn{\alpha_{m,T+1}}}.
+#'     The last row is for \eqn{\alpha_{m,T+1}}.}
 #'   \item{If \code{to_return=="terms"}:}{a size ((n_obs-p)x1) numeric vector containing the terms \eqn{l_{t}}.}
 #'   \item{if \code{to_return=="loglik_and_mw"}:}{a list of two elements. The first element contains the log-likelihood value and the
 #'     second element contains the mixing weights.}
 #'   \item{If \code{to_return=="regime_cmeans"}:}{an \code{[T-p, d, M]} array containing the regimewise conditional means
 #'    (the first p values are used as the initial values).}
+#'    \item{If \code{to_return=="regime_ccovs"}:}{an \code{[d, d, T-p, M]} array containing the regimewise conditional
+#'    covariance matrices (the first p values are used as the initial values). The index \code{[ , , t, m]} gives the time
+#'    \code{t} conditional covariance matrix for the regime \code{m}.}
 #'   \item{If \code{to_return=="total_cmeans"}:}{a \code{[T-p, d]} matrix containing the conditional means of the process
 #'    (the first p values are used as the initial values).}
 #'   \item{If \code{to_return=="total_ccov"}:}{an \code{[d, d, T-p]} array containing the conditional covariance matrices of the process
 #'    (the first p values are used as the initial values).}
-#'  }
+#'   \item{If \code{to_return=="arch_scalars"}:}{a \code{[T-p, M]} matrix containing the regimewise arch scalars
+#'    multiplying error term covariance matrix in the conditional covariance matrix of the regime. For GMVAR type regimes, these
+#'    are all ones (the first p values are used as the initial values).}
+#'   \item{if \code{to_return=="loglik_mw_archscalars"}:}{a list of three elements. The first element contains the log-likelihood value, the
+#'     second element contains the mixing weights, the third element contains the arch scalars
+#'     (this is used in \code{quantile_residuals_int}).}
 #' @references
 #'  \itemize{
 #'    \item Kalliovirta L., Meitz M. and Saikkonen P. 2016. Gaussian mixture vector autoregression.
@@ -142,13 +159,17 @@
 #'          \emph{Statistics and Probability Letters}, \strong{124}, 92-96.
 #'    \item Virolainen S. 2020. Structural Gaussian mixture vector autoregressive model. Unpublished working
 #'          paper, available as arXiv:2007.04713.
+#'    \item Virolainen S. 2021. Gaussian and Student's t mixture vector autoregressive model. Unpublished working
+#'      paper, available as arXiv:2109.13648.
 #'  }
 #' @keywords internal
 
-loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrization=c("intercept", "mean"), constraints=NULL,
-                              same_means=NULL, structural_pars=NULL,
-                              to_return=c("loglik", "mw", "mw_tplus1", "loglik_and_mw", "terms", "regime_cmeans", "total_cmeans", "total_ccovs"),
-                              check_params=TRUE, minval=NULL, stat_tol=1e-3, posdef_tol=1e-8) {
+loglikelihood_int <- function(data, p, M, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), conditional=TRUE, parametrization=c("intercept", "mean"),
+                              constraints=NULL, same_means=NULL, structural_pars=NULL,
+                              to_return=c("loglik", "mw", "mw_tplus1", "loglik_and_mw", "terms",
+                                          "regime_cmeans", "regime_ccovs", "total_cmeans", "total_ccovs",
+                                          "arch_scalars", "loglik_mw_archscalars"),
+                              check_params=TRUE, minval=NULL, stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
 
   # Compute required values
   epsilon <- round(log(.Machine$double.xmin) + 10) # Logarithm of the smallest value that can be handled normally
@@ -158,9 +179,11 @@ loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrizat
   to_return <- match.arg(to_return)
 
   # Collect parameter values
+  model <- match.arg(model)
   parametrization <- match.arg(parametrization)
   check_same_means(parametrization=parametrization, same_means=same_means)
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, model=model, constraints=constraints,
+                                    same_means=same_means, structural_pars=structural_pars) # All constraints are expanded and removed from the parameter vector
   W_constraints <- structural_pars$W
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   if(parametrization == "intercept") {
@@ -171,17 +194,31 @@ loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrizat
   all_A <- pick_allA(p=p, M=M, d=d, params=params, structural_pars=structural_pars) # A_{m,i}, m=1,...,M, i=1,..,p
   all_Omega <- pick_Omegas(p=p, M=M, d=d, params=params, structural_pars=structural_pars) # Omega_m
   all_boldA <- form_boldA(p=p, M=M, d=d, all_A=all_A) # The 'bold A' for each m=1,..,M, Lütkepohl 2005, eq.(2.1.8)
-  alphas <- pick_alphas(p=p, M=M, d=d, params=params) # Mixing weight parameters
+  alphas <- pick_alphas(p=p, M=M, d=d, params=params, model=model) # Mixing weight parameters
+  all_df <- pick_df(M=M, params=params, model=model) # Degrees of freedom parameters (numeric(0) for GMVAR models)
 
   # Check that the parameter vector lies in the parameter space (excluding indentifiability)
   if(check_params) {
-    if(!in_paramspace_int(p=p, M=M, d=d, params=params, all_boldA=all_boldA, alphas=alphas, all_Omega=all_Omega,
-                          W_constraints=W_constraints, stat_tol=stat_tol, posdef_tol=posdef_tol)) {
+    if(!in_paramspace_int(p=p, M=M, d=d, params=params, model=model, all_boldA=all_boldA, alphas=alphas, all_Omega=all_Omega,
+                          W_constraints=W_constraints, stat_tol=stat_tol, posdef_tol=posdef_tol, df_tol=df_tol)) {
       return(minval)
     }
   }
 
-  # An i:th row denotes the vector \bold{y_{i-1}} = (y_{i-1}',...,y_{i-p}') (dpx1),
+  if(model == "GMVAR") {
+    M1 <- M
+    M2 <- 0
+  } else if(model == "StMVAR") {
+    M1 <- 0
+    M2 <- M
+  } else { # model == "G-StMVAR"
+    M1 <- M[1]
+    M2 <- M[2]
+  }
+  M <- sum(M) # The total number of mixture components
+
+
+  # i:th row denotes the vector \bold{y_{i-1}} = (y_{i-1},...,y_{i-p}) (dpx1),
   # assuming the observed data is y_{-p+1},...,y_0,y_1,...,y_{T}
   Y <- reform_data(data, p)
 
@@ -195,20 +232,39 @@ loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrizat
 
   # Calculate the covariance matrices Sigma_{m,p} (Lutkepohl 2005, eq. (2.1.39) or the algorithm proposed by McElroy 2017)
   Sigmas <- get_Sigmas(p=p, M=M, d=d, all_A=all_A, all_boldA=all_boldA, all_Omega=all_Omega) # Store the (dpxdp) covariance matrices
-  chol_Sigmas <- array(NA, dim=c(d*p, d*p, M))
+  inv_Sigmas <- array(dim=c(d*p, d*p, M)) # Only used for StMVAR type regimes
+  chol_Sigmas <- array(dim=c(d*p, d*p, M))
   for(m in 1:M) {
-    chol_Sigmas[, , m] <- chol(Sigmas[, , m]) # Take Cholesky here to avoid unnecessary warnings from mvnfast::dmvn
+    chol_Sigmas[, , m] <- chol(Sigmas[, , m]) # Take Cholesky here to avoid unnecessary warnings from mvnfast::dmvn, also used in inverting for m > M1
+    if(m > M1) {
+      inv_Sigmas[, , m] <- chol2inv(chol_Sigmas[, , m])
+    }
   }
 
-  # Calculate the dp-dimensional multinormal densities (KMS 2016, eq.(6)), i:th row for index i-1 etc, m:th column for m:th component
-  # Calculated in logarithm because same values may be too close to zero for machine accuracy
-  log_mvnvalues <- vapply(1:M, function(m) mvnfast::dmvn(X=Y, mu=rep(mu[,m], p), sigma=chol_Sigmas[, , m], log=TRUE, ncores=1, isChol=TRUE), numeric(T_obs + 1))
+  # Calculate the dp-dimensional multinormal densities (KMS 2016, eq.(6)) or log Students t densities (Virolainen 2021, eq. (3.4)):
+  # i:th row for index i-1 etc, m:th column for m:th component.
+  # We calculate in logarithm because the non-log values may be too close to zero for machine accuracy (if they are too close to zero
+  # for all regimes and computer handles them as zero, we would divide by zero when calculating the mixing weights)
+  log_mvdvalues <- matprods <- matrix(nrow=n_obs - p + 1, ncol=M) # The quadratic forms in Student's t density
+  if(M1 > 0) { # Multinormals
+    log_mvdvalues[,1:M1] <- vapply(1:M1, function(m) mvnfast::dmvn(X=Y, mu=rep(mu[,m], p), sigma=chol_Sigmas[, , m],
+                                                                   log=TRUE, ncores=1, isChol=TRUE), numeric(T_obs + 1))
+  }
+  if(M2 > 0) { # Multistudents
+    for(m in (M1 + 1):M) { # Go through the StMVAR type regimes
+      tmp_mat <- Y - matrix(rep(mu[,m], p), nrow=nrow(Y), ncol=ncol(Y), byrow=TRUE)
+      matprods[,m] <- rowSums(tmp_mat%*%inv_Sigmas[, , m]*tmp_mat)
+      logC <- lgamma(0.5*(d*p + all_df[m - M1])) - 0.5*d*p*log(base::pi) - 0.5*d*p*log(all_df[m - M1] - 2) - lgamma(0.5*all_df[m - M1])
+      log_det_Sigma <- 2*log(prod(diag(chol_Sigmas[, , m])))  #log(det(Sigmas[, , m]))
+      log_mvdvalues[,m] <- logC - 0.5*log_det_Sigma - 0.5*(d*p + all_df[m - M1])*log(1 + matprods[,m]/(all_df[m - M1] - 2))
+    }
+  }
 
   ## Calculate the mixing weights alpha_{m,t} (KMS 2016, eq.(7))
   if(to_return != "mw_tplus1") {
-    log_mvnvalues <- log_mvnvalues[1:T_obs, , drop=FALSE] # alpha_mt uses y_{t-1} so the last row is not needed
+    log_mvdvalues <- log_mvdvalues[1:T_obs, , drop=FALSE] # alpha_mt uses y_{t-1} so the last row is not needed
   }
-  alpha_mt_and_l_0 <- get_alpha_mt(M=M, log_mvnvalues=log_mvnvalues, alphas=alphas,
+  alpha_mt_and_l_0 <- get_alpha_mt(M=M, log_mvdvalues=log_mvdvalues, alphas=alphas,
                                    epsilon=epsilon, conditional=conditional, also_l_0=TRUE)
   alpha_mt <- alpha_mt_and_l_0$alpha_mt
   l_0 <- alpha_mt_and_l_0$l_0 # The first term in the exact log-likelihood function (=0 for conditional)
@@ -217,37 +273,77 @@ loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrizat
     return(alpha_mt)
   }
 
-  # Calculate the conditional means mu_{m,t} (KMS 2016, Condition 1 (a)).
+  # Calculate the conditional means mu_{m,t} (KMS 2016, Condition 1 (a))
   # The dimensions of mu_mt will be: [t, p, m]
   all_A2 <- array(all_A, dim=c(d, d*p, M)) # cbind coefficient matrices of each component: m:th component is obtained at [, , m]
   Y2 <- Y[1:T_obs,] # Last row is not needed because mu_mt uses lagged values
   mu_mt <- array(vapply(1:M, function(m) t(all_phi0[, m] + tcrossprod(all_A2[, , m], Y2)), numeric(d*T_obs)), dim=c(T_obs, d, M)) # [, , m]
 
+  # For the StMVAR type regimes, calculate the time-varying ARC-type scalar that multiplies the conditional covariance matrix:
+  # i:th row calculated from y_{i-1} observation when the indexing is y_{-p+1},..,y_0,y_1,...,y_T - note that the last row of arch_scalars is for time T+1
+  arch_scalars <- matrix(1, nrow=nrow(matprods) - 1, ncol=M) # The arch-scalars are all 1 for the GMVAR type regimes
+  if(M2 > 0) {
+    matprods0 <- as.matrix(matprods[1:(nrow(matprods) - 1), (M1 + 1):M]) # The last row is not needed because for the time t, we use the previous value y_{t-1}
+    tmp_mat <- matrix(all_df, nrow=nrow(matprods0), ncol=ncol(matprods0), byrow=TRUE)
+    arch_scalars[,(M1 + 1):M] <- (tmp_mat - 2 + matprods0)/(tmp_mat - 2 + d*p)
+  }
+
+
   if(to_return == "regime_cmeans") {
     return(mu_mt)
+  } else if(to_return == "arch_scalars") {
+    return(arch_scalars)
   } else if(to_return == "total_cmeans") { # KMS 2016, eq.(3)
     return(matrix(rowSums(vapply(1:M, function(m) alpha_mt[,m]*mu_mt[, , m], numeric(d*T_obs))), nrow=T_obs, ncol=d, byrow=FALSE))
+  } else if(to_return == "regime_ccovs") {
+    regime_ccovs <- array(vapply(1:M, function(m) t(matrix(all_Omega[, , m], nrow=nrow(arch_scalars), ncol=d^2, byrow=TRUE)*arch_scalars[, m]),
+                                 numeric(d*d*nrow(arch_scalars))), dim=c(d, d, nrow(arch_scalars), M))
+    return(regime_ccovs)
   } else if(to_return == "total_ccovs") { # KMS 2016, eq.(4)
-    first_term <- array(rowSums(vapply(1:M, function(m) rep(alpha_mt[, m], each=d*d)*as.vector(all_Omega[, , m]), numeric(d*d*T_obs))), dim=c(d, d, T_obs))
-    sum_alpha_mu <- matrix(rowSums(vapply(1:M, function(m) alpha_mt[, m]*mu_mt[, , m], numeric(d*T_obs))), nrow=T_obs, ncol=d, byrow=FALSE)
-    second_term <- array(rowSums(vapply(1:M, function(m) rep(alpha_mt[, m], each=d*d)*as.vector(vapply(1:nrow(alpha_mt), function(i1) tcrossprod((mu_mt[, , m] - sum_alpha_mu)[i1,]),
+    first_term <- array(rowSums(vapply(1:M, function(m) rep(alpha_mt[, m]*arch_scalars[, m], each=d*d)*as.vector(all_Omega[, , m]),
+                                       numeric(d*d*T_obs))), dim=c(d, d, T_obs))
+    sum_alpha_mu <- matrix(rowSums(vapply(1:M, function(m) alpha_mt[, m]*mu_mt[, , m],
+                                          numeric(d*T_obs))), nrow=T_obs, ncol=d, byrow=FALSE)
+    second_term <- array(rowSums(vapply(1:M, function(m) rep(alpha_mt[, m], each=d*d)*as.vector(vapply(1:nrow(alpha_mt),
+                                                                                                       function(i1) tcrossprod((mu_mt[, , m] - sum_alpha_mu)[i1,]),
                                                                                                        numeric(d*d))), numeric(d*d*T_obs))), dim=c(d, d, T_obs))
     return(first_term + second_term)
   }
 
-  # Calculate the second term of the log-likelihood (KMS 2016 eq.(10))
-  dat <- data[(p + 1):n_obs,] # Initial values are not used here
-  mvn_vals <- vapply(1:M, function(m) mvnfast::dmvn(X=dat - mu_mt[, , m], mu=rep(0, times=d), sigma=all_Omega[, , m], log=FALSE, ncores=1, isChol=FALSE), numeric(T_obs))
-  weighted_mvn <- rowSums(alpha_mt*mvn_vals)
-  weighted_mvn[weighted_mvn == 0] <- exp(epsilon)
-  l_t <- log(weighted_mvn)
+  # Calculate the second term of the log-likelihood (KMS 2016 eq.(10)), also see Virolainen (2021) for the StMVAR type regimes
+  dat <- data[(p + 1):n_obs,] # Initial values are not used here (conditional means and variances are already calculated)
+  mvd_vals <- matrix(nrow=T_obs, ncol=M)
+  if(M1 > 0) { # GMVAR type regimes
+    mvd_vals[,1:M1] <- vapply(1:M1, function(m) mvnfast::dmvn(X=dat - mu_mt[, , m], mu=rep(0, times=d), sigma=all_Omega[, , m], log=FALSE, ncores=1, isChol=FALSE), numeric(T_obs))
+  }
+  if(M2 > 0) { # StMVAR type regimes
+    for(m in (M1 + 1):M) {
+      df_m <- all_df[m - M1] + d*p # Degrees of freedom in regime m
+      chol_Omega_m <- chol(all_Omega[, , m]) # Faster determinant and matrix inversion useing Cholesky decomposition
+      det_Omega_m <- prod(diag(chol_Omega_m))^2
+      inv_Omega_m <- chol2inv(chol_Omega_m)
+      # Below, we calculate the d-dimensional conditional t-densities for the regime m, for t=1,...,T.
+      tmp_mat <- dat - mu_mt[, , m]
+      mvd_vals[, m] <- exp(lgamma(0.5*(d + df_m)) - 0.5*d*log(base::pi) - 0.5*d*log(df_m - 2) - lgamma(0.5*df_m))*arch_scalars[, m]^(-d/2)*1/sqrt(det_Omega_m)*(1 + rowSums(tmp_mat%*%inv_Omega_m*tmp_mat)/(arch_scalars[, m]*(df_m - 2)))^(-0.5*(d + df_m))
+    }
+  }
+
+  weighted_mvd <- rowSums(alpha_mt*mvd_vals)
+  weighted_mvd[weighted_mvd == 0] <- exp(epsilon)
+  l_t <- log(weighted_mvd)
+  loglik <- l_0 + sum(l_t)
 
   if(to_return == "terms") {
      return(l_t)
   } else if(to_return == "loglik_and_mw") {
-    return(list(loglik=l_0 + sum(l_t), mw=alpha_mt))
-  } else {
-    return(l_0 + sum(l_t))
+    return(list(loglik=loglik,
+                mw=alpha_mt))
+  } else if(to_return == "loglik_mw_archscalars") {
+    return(list(loglik=loglik,
+                mw=alpha_mt,
+                arch_scalars=arch_scalars))
+  } else { # to_return == "loglik"
+    return(loglik)
   }
 }
 
@@ -259,33 +355,33 @@ loglikelihood_int <- function(data, p, M, params, conditional=TRUE, parametrizat
 #'   the mixing weights.
 #'
 #' @inheritParams loglikelihood_int
-#' @param log_mvnvalues \eqn{T x M} matrix containing the log multivariate normal densities.
+#' @param log_mvdvalues \eqn{T x M} matrix containing the log multivariate normal densities.
 #' @param alphas \eqn{M x 1} vector containing the mixing weight pa
 #' @param epsilon the smallest number such that its exponent is wont classified as numerically zero
 #'   (around \code{-698} is used).
 #' @param also_l_0 return also l_0 (the first term in the exact log-likelihood function)?
 #' @details Note that we index the time series as \eqn{-p+1,...,0,1,...,T} as in Kalliovirta et al. (2016).
-#' @return Returns the mixing weights a matrix of the same dimension as \code{log_mvnvalues} so
+#' @return Returns the mixing weights a matrix of the same dimension as \code{log_mvdvalues} so
 #'   that the t:th row is for the time point t and m:th column is for the regime m.
 #' @inherit in_paramspace_int references
 #' @seealso \code{\link{loglikelihood_int}}
 #' @keywords internal
 
-get_alpha_mt <- function(M, log_mvnvalues, alphas, epsilon, conditional, also_l_0=FALSE) {
+get_alpha_mt <- function(M, log_mvdvalues, alphas, epsilon, conditional, also_l_0=FALSE) {
   if(M == 1) {
-    if(!is.matrix(log_mvnvalues)) log_mvnvalues <- as.matrix(log_mvnvalues) # Possibly many time points but only one regime
-    alpha_mt <- as.matrix(rep(1, nrow(log_mvnvalues)))
+    if(!is.matrix(log_mvdvalues)) log_mvdvalues <- as.matrix(log_mvdvalues) # Possibly many time points but only one regime
+    alpha_mt <- as.matrix(rep(1, nrow(log_mvdvalues)))
   } else {
-    if(!is.matrix(log_mvnvalues)) log_mvnvalues <- t(as.matrix(log_mvnvalues)) # Only one time point but multiple regimes
+    if(!is.matrix(log_mvdvalues)) log_mvdvalues <- t(as.matrix(log_mvdvalues)) # Only one time point but multiple regimes
 
-    log_mvnvalues_orig <- log_mvnvalues
-    small_logmvns <- log_mvnvalues < epsilon
+    log_mvdvalues_orig <- log_mvdvalues
+    small_logmvns <- log_mvdvalues < epsilon
     if(any(small_logmvns)) {
       # If too small or large non-log-density values are present (i.e., that would yield -Inf or Inf),
       # we replace them with ones that are not too small or large but imply the same mixing weights
       # up to negligible numerical tolerance.
       which_change <- rowSums(small_logmvns) > 0 # Which rows contain too small  values
-      to_change <- log_mvnvalues[which_change, , drop=FALSE]
+      to_change <- log_mvdvalues[which_change, , drop=FALSE]
       largest_vals <- do.call(pmax, split(to_change, f=rep(1:ncol(to_change), each=nrow(to_change)))) # The largest values of those rows
       diff_to_largest <- to_change - largest_vals # Differences to the largest value of the row
 
@@ -293,11 +389,11 @@ get_alpha_mt <- function(M, log_mvnvalues, alphas, epsilon, conditional, also_l_
       # is smaller than epsilon, replace the with epsilon. The results are then the new log_mvn values.
       diff_to_largest[diff_to_largest < epsilon] <- epsilon
 
-      # Replace the old log_mvnvalues with the new ones
-      log_mvnvalues[which_change,] <- diff_to_largest
+      # Replace the old log_mvdvalues with the new ones
+      log_mvdvalues[which_change,] <- diff_to_largest
     }
 
-    mvnvalues <- exp(log_mvnvalues)
+    mvnvalues <- exp(log_mvdvalues)
     denominator <- as.vector(mvnvalues%*%alphas)
     alpha_mt <- (mvnvalues/denominator)%*%diag(alphas)
   }
@@ -307,10 +403,10 @@ get_alpha_mt <- function(M, log_mvnvalues, alphas, epsilon, conditional, also_l_
     # First term of the exact log-likelihood (Kalliovirta et al. 2016, eq.(9))
     l_0 <- 0
     if(M == 1 && conditional == FALSE) {
-      l_0 <- log_mvnvalues[1,]
+      l_0 <- log_mvdvalues[1,]
     } else if(M > 1 && conditional == FALSE) {
-      if(any(log_mvnvalues_orig[1,] < epsilon)) { # Need to use Brobdingnag
-        l_0 <- log(Reduce("+", lapply(1:M, function(i1) alphas[i1]*exp(Brobdingnag::as.brob(log_mvnvalues_orig[1, i1])))))
+      if(any(log_mvdvalues_orig[1,] < epsilon)) { # Need to use Brobdingnag
+        l_0 <- log(Reduce("+", lapply(1:M, function(i1) alphas[i1]*exp(Brobdingnag::as.brob(log_mvdvalues_orig[1, i1])))))
       } else {
         l_0 <- log(sum(alphas*mvnvalues[1,]))
       }
@@ -321,17 +417,18 @@ get_alpha_mt <- function(M, log_mvnvalues, alphas, epsilon, conditional, also_l_
 }
 
 
-#' @title Compute log-likelihood of a GMVAR model using parameter vector
+
+#' @title Compute log-likelihood of a GMVAR, StMVAR, or G-StMVAR model using parameter vector
 #'
-#' @description \code{loglikelihood} computes log-likelihood of a GMVAR model using parameter vector
-#'   instead of an object of class 'gmvar'. Exists for convenience if one wants to for example
-#'   employ other estimation algorithms than the ones used in \code{fitGMVAR}. Use \code{minval} to
+#' @description \code{loglikelihood} computes log-likelihood of a GMVAR, StMVAR, or G-StMVAR model using parameter vector
+#'   instead of an object of class 'gsmvar'. Exists for convenience if one wants to for example
+#'   employ other estimation algorithms than the ones used in \code{fitGSMVAR}. Use \code{minval} to
 #'   control what happens when the parameter vector is outside the parameter space.
 #'
 #' @inheritParams loglikelihood_int
 #' @return Returns log-likelihood if \code{params} is in the parameters space and \code{minval} if not.
 #' @inherit loglikelihood_int details references
-#' @seealso \code{\link{fitGMVAR}}, \code{\link{GMVAR}}, \code{\link{calc_gradient}}
+#' @seealso \code{\link{fitGSMVAR}}, \code{\link{GSMVAR}}, \code{\link{calc_gradient}}
 #' @examples
 #' # GMVAR(2, 2), d=2 model;
 #' params22 <- c(0.36, 0.121, 0.223, 0.059, -0.151, 0.395, 0.406, -0.005,
@@ -347,27 +444,29 @@ get_alpha_mt <- function(M, log_mvnvalues, alphas, epsilon, conditional, also_l_
 #' loglikelihood(data=gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
 #' @export
 
-loglikelihood <- function(data, p, M, params, conditional=TRUE, parametrization=c("intercept", "mean"), constraints=NULL,
-                          same_means=NULL, structural_pars=NULL, minval=NA, stat_tol=1e-3, posdef_tol=1e-8) {
-  if(!all_pos_ints(c(p, M))) stop("Arguments p and M must be positive integers")
+loglikelihood <- function(data, p, M, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), conditional=TRUE, parametrization=c("intercept", "mean"),
+                          constraints=NULL, same_means=NULL, structural_pars=NULL, minval=NA,
+                          stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
   parametrization <- match.arg(parametrization)
+  model <- match.arg(model)
   check_same_means(parametrization=parametrization, same_means=same_means)
   data <- check_data(data, p)
   d <- ncol(data)
+  check_pMd(p=p, M=M, d=d, model=model)
   check_constraints(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
-  if(length(params) != n_params(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
+  if(length(params) != n_params(p=p, M=M, d=d, model=model, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
     stop("Parameter vector has wrong dimension")
   }
-  loglikelihood_int(data=data, p=p, M=M, params=params, conditional=conditional, parametrization=parametrization,
+  loglikelihood_int(data=data, p=p, M=M, params=params, model=model, conditional=conditional, parametrization=parametrization,
                     constraints=constraints, same_means=same_means, structural_pars=structural_pars, to_return="loglik",
-                    check_params=TRUE, minval=minval, stat_tol=stat_tol, posdef_tol=posdef_tol)
+                    check_params=TRUE, minval=minval, stat_tol=stat_tol, posdef_tol=posdef_tol, df_tol=df_tol)
 }
 
 
-#' @title Compute conditional moments of a GMVAR model
+#' @title Compute conditional moments of a GMVAR, StMVAR, or G-StMVAR model
 #'
 #' @description \code{loglikelihood} compute conditional regimewise means, conditional means, and conditional covariance matrices
-#'  of a GMVAR model.
+#'  of a GMVAR, StMVAR, or G-StMVAR model.
 #'
 #' @inheritParams loglikelihood_int
 #' @param to_return should the regimewise conditional means, total conditional means, or total conditional covariance matrices
@@ -375,14 +474,18 @@ loglikelihood <- function(data, p, M, params, conditional=TRUE, parametrization=
 #' @details The first p values are used as the initial values, and by conditional we mean conditioning on the past. Formulas
 #'   for the conditional means and covariance matrices are given in equations (3) and (4) of KMS (2016).
 #' @return
-#'  \describe{
 #'   \item{If \code{to_return=="regime_cmeans"}:}{an \code{[T-p, d, M]} array containing the regimewise conditional means
-#'    (the first p values are used as the initial values).}
+#'     (the first p values are used as the initial values).}
+#'   \item{If \code{to_return=="regime_ccovs"}:}{an \code{[d, d, T-p, M]} array containing the regimewise conditional
+#'     covariance matrices (the first p values are used as the initial values). The index \code{[ , , t, m]} gives the time
+#'     \code{t} conditional covariance matrix for the regime \code{m}.}
 #'   \item{If \code{to_return=="total_cmeans"}:}{a \code{[T-p, d]} matrix containing the conditional means of the process
-#'    (the first p values are used as the initial values).}
+#'     (the first p values are used as the initial values).}
 #'   \item{If \code{to_return=="total_ccov"}:}{an \code{[d, d, T-p]} array containing the conditional covariance matrices of the process
-#'    (the first p values are used as the initial values).}
-#'  }
+#'     (the first p values are used as the initial values).}
+#'   \item{If \code{to_return=="arch_scalars"}:}{a \code{[T-p, M]} matrix containing the regimewise arch scalars
+#'    multiplying error term covariance matrix in the conditional covariance matrix of the regime. For GMVAR type regimes, these
+#'    are all ones (the first p values are used as the initial values).}
 #' @inherit loglikelihood_int references
 #' @family moment functions
 #' @examples
@@ -395,22 +498,23 @@ loglikelihood <- function(data, p, M, params, conditional=TRUE, parametrization=
 #' cond_moments(data=gdpdef, p=2, M=2, params=params22, to_return="total_ccovs")
 #' @export
 
-cond_moments <- function(data, p, M, params, parametrization=c("intercept", "mean"), constraints=NULL, same_means=NULL,
-                         structural_pars=NULL, to_return=c("regime_cmeans", "total_cmeans", "total_ccovs"),
-                         stat_tol=1e-3, posdef_tol=1e-8) {
-  if(!all_pos_ints(c(p, M))) stop("Arguments p and M must be positive integers")
+cond_moments <- function(data, p, M, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), parametrization=c("intercept", "mean"), constraints=NULL, same_means=NULL,
+                         structural_pars=NULL, to_return=c("regime_cmeans", "regime_ccovs", "total_cmeans", "total_ccovs", "arch_scalars"), minval=NA,
+                         stat_tol=1e-3, posdef_tol=1e-8, df_tol=1e-8) {
   parametrization <- match.arg(parametrization)
-  check_same_means(parametrization=parametrization, same_means=same_means)
+  model <- match.arg(model)
   to_return <- match.arg(to_return)
+  check_same_means(parametrization=parametrization, same_means=same_means)
   data <- check_data(data, p)
   d <- ncol(data)
-  check_constraints(p=p, M=M, d=d, constraints=constraints, structural_pars=structural_pars)
-  if(length(params) != n_params(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
+  check_pMd(p=p, M=M, d=d, model=model)
+  check_constraints(p=p, M=M, d=d, constraints=constraints, same_means=same_means, structural_pars=structural_pars)
+  if(length(params) != n_params(p=p, M=M, d=d, model=model, constraints=constraints, same_means=same_means, structural_pars=structural_pars)) {
     stop("Parameter vector has wrong dimension")
   }
-  loglikelihood_int(data=data, p=p, M=M, params=params, conditional=TRUE, parametrization=parametrization,
-                    constraints=constraints, same_means=same_means, structural_pars=structural_pars, to_return=to_return, check_params=TRUE,
-                    minval=NA, stat_tol=stat_tol, posdef_tol=posdef_tol)
+  loglikelihood_int(data=data, p=p, M=M, params=params, model=model, parametrization=parametrization,
+                    constraints=constraints, same_means=same_means, structural_pars=structural_pars, to_return=to_return,
+                    check_params=TRUE, minval=minval, stat_tol=stat_tol, posdef_tol=posdef_tol, df_tol=df_tol)
 }
 
 
